/*
 */

package com.ak.osgi.service.test;

import java.lang.reflect.Method;
import java.rmi.RemoteException;
import java.rmi.server.RemoteObject;

import org.knopflerfish.service.log.LogRef;
import org.osgi.framework.BundleActivator;
import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceReference;

import com.ak.osgi.service.rmi.interfaces.demo1.RemoteDemo1;
import com.ak.osgi.service.rmi.interfaces.demo2.RemoteDemo2;
import com.ak.osgi.service.rmi.utility.DebugLog;

/**
 * Activator which creates and registers a Demo1 service.
 */
public class Activator implements BundleActivator
{
//  private RemoteOSGiFramework remoteContext;
  private RemoteDemo1 demo1;
  private RemoteDemo2 demo2;
  static LogRef log;

  public void start( BundleContext bc )
  {
    System.out.println( "start " + getClass().getName() );
    Activator.log = new LogRef( bc );
    log.info( "start " + getClass().getName() );

    ServiceReference sref = bc.getServiceReference( RemoteDemo1.class.getName() );
    if( sref == null )
    {
      System.out.println( "can not get service reference for Remote Demo 1 : sref = null !!!" );
    }
    else
    {
      RemoteObject obj = (RemoteObject)bc.getService( sref );
      DebugLog.dumpClassInfo( obj );
      DebugLog.logRemoteInstance( log, "Demo1", obj );
      demo1 = (RemoteDemo1)obj;
      if( demo1 == null )
      {
        System.out.println( "can not get remote service: demo1 = null !!!" );
      }
      else
      {
//        dumpClassInfo( demo1 );
        try
        {
          System.out.println( " 2 + 3 = " + demo1.add( 2, 3 ) );
          System.out.println( " 5 - 3 = " + demo1.substract( 5, 3 ) );
        }
        catch( RemoteException e )
        {
          log.error( "can not use remote service" );
        }
        bc.ungetService( sref );
      }
    }
    
    sref = bc.getServiceReference( RemoteDemo2.class.getName() );
    if( sref == null )
    {
      System.out.println( "can not get service reference for Remote Demo 2 : sref = null !!!" );
    }
    else
    {
      demo2 = (RemoteDemo2)bc.getService( sref );
      if( demo2 == null )
      {
        System.out.println( "can not get remote service: demo2 = null !!!" );
      }
      else
      {
        try
        {
          System.out.println( " 5 * 3 = " + demo2.multiply( 5, 3 ) );
          System.out.println( " 10 / 2 = " + demo2.divide( 10, 2 ) );
        }
        catch( RemoteException e )
        {
          log.error( "can not use remote service" );
        }
      }
    }
//    ServiceReference [] srefs = null;
//    try
//    {
//      String rmiServiceFilter = "(" + RmiConstants.KEY_RMISERVICE_REM + "=*)";
//      srefs = bc.getServiceReferences( RemoteOSGiFramework.class.getName(), rmiServiceFilter );
//      System.out.println( "ref count = " + srefs.length );
//    }
//    catch( InvalidSyntaxException e1 )
//    {
//      e1.printStackTrace();
//    }
//    if( srefs == null || srefs.length < 1 || srefs[0] == null )
//    {
//      System.out
//          .println( "can not get service reference for RemoteOSGiFramework : sref = null !!!" );
//    }
//    else
//    {
//      remoteContext = (RemoteOSGiFramework)bc.getService( srefs[0] );
//      if( remoteContext == null )
//      {
//        System.out.println( "can not get 'RemoteOSGiFramework' service: remoteContext = null !!!" );
//      }
//      else
//      {
//        dumpClassInfo( remoteContext );
//        //      remoteContext = (RemoteOSGiFramework)bc.getService( srefs[1] );
//        //      dumpClassInfo( remoteContext );
//        BundleShortInfoBean binfo;
//        BundleShortInfoBean [] binfos;
//        try
//        {
//          binfo = remoteContext.getBundle( 1 );
//          System.out.println( binfo.getName() + " [" + binfo.getBundleId() + "] " + " loc : "
//              + binfo.getLocation() );
//          binfos = remoteContext.getBundles();
//          for( int i = 0; i < binfos.length; i++ )
//          {
//            System.out.println( binfos[i].getName() + " [" + binfos[i].getBundleId() + "]"
//                + " loc: " + binfos[i].getLocation() );
//          }
//        }
//        catch( RemoteException e )
//        {
//          log.error( "can not use remote service" );
//        }
//      }
//    }
  }

  public void stop( BundleContext bc )
  {
    log.info( "stop " + getClass().getName() );
    demo1 = null;
    demo2 = null;
  }

  private void dumpClassInfo( Object o )
  {
    System.out.println( " -- object is " + ((o == null) ? "null" : "not null") );
    if( o != null )
    {
      Class stub_class = o.getClass();
      dumpInterfaceInfo( stub_class );
    }
  }
  
  private void dumpInterfaceInfo( Class clss )
  {
    ClassLoader loader = clss.getClassLoader();
    Class [] ifaces = clss.getInterfaces();
    Method [] methods = clss.getMethods();
    Package pck = clss.getPackage();
    System.out.println( "\n -- class name : " + clss.getName() 
        + "\n -- package name : " + ((pck==null)?"null":pck.getName()) 
        + "\n -- classLoader name : " + ((loader==null)?"null":loader.getClass().getName()) 
        );
    System.out.println( "  Interfaces : " );
    for( int i = 0; i < ifaces.length; i++ )
      System.out.println( i + ">> " + ifaces[i].getName() );
//      System.out.println( "  Methods : " );
//      for( int i = 0; i < methods.length; i++ )
//        System.out.println( i + ">> " + methods[i].getName() + "();" );
  }
}
